@@ -93,7 +93,7 @@ gdk_cursor_type_get_type (void)
       { GDK_CURSOR_IS_PIXMAP, "GDK_CURSOR_IS_PIXMAP", "cursor-is-pixmap" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkCursorType", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkCursorType"), values);
   }
   return etype;
 }
@@ -113,7 +113,7 @@ gdk_drag_action_get_type (void)
       { GDK_ACTION_ASK, "GDK_ACTION_ASK", "ask" },
       { 0, NULL, NULL }
     };
-    etype = g_flags_register_static ("GdkDragAction", values);
+    etype = g_flags_register_static (g_intern_static_string ("GdkDragAction"), values);
   }
   return etype;
 }
@@ -132,7 +132,7 @@ gdk_drag_protocol_get_type (void)
       { GDK_DRAG_PROTO_LOCAL, "GDK_DRAG_PROTO_LOCAL", "local" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkDragProtocol", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkDragProtocol"), values);
   }
   return etype;
 }
@@ -149,7 +149,7 @@ gdk_filter_return_get_type (void)
       { GDK_FILTER_REMOVE, "GDK_FILTER_REMOVE", "remove" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkFilterReturn", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkFilterReturn"), values);
   }
   return etype;
 }
@@ -195,9 +195,10 @@ gdk_event_type_get_type (void)
       { GDK_WINDOW_STATE, "GDK_WINDOW_STATE", "window-state" },
       { GDK_SETTING, "GDK_SETTING", "setting" },
       { GDK_OWNER_CHANGE, "GDK_OWNER_CHANGE", "owner-change" },
+      { GDK_GRAB_BROKEN, "GDK_GRAB_BROKEN", "grab-broken" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkEventType", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkEventType"), values);
   }
   return etype;
 }
@@ -231,7 +232,7 @@ gdk_event_mask_get_type (void)
       { GDK_ALL_EVENTS_MASK, "GDK_ALL_EVENTS_MASK", "all-events-mask" },
       { 0, NULL, NULL }
     };
-    etype = g_flags_register_static ("GdkEventMask", values);
+    etype = g_flags_register_static (g_intern_static_string ("GdkEventMask"), values);
   }
   return etype;
 }
@@ -246,7 +247,7 @@ gdk_visibility_state_get_type (void)
       { GDK_VISIBILITY_FULLY_OBSCURED, "GDK_VISIBILITY_FULLY_OBSCURED", "fully-obscured" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkVisibilityState", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkVisibilityState"), values);
   }
   return etype;
 }
@@ -262,7 +263,7 @@ gdk_scroll_direction_get_type (void)
       { GDK_SCROLL_RIGHT, "GDK_SCROLL_RIGHT", "right" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkScrollDirection", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkScrollDirection"), values);
   }
   return etype;
 }
@@ -280,7 +281,7 @@ gdk_notify_type_get_type (void)
       { GDK_NOTIFY_UNKNOWN, "GDK_NOTIFY_UNKNOWN", "unknown" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkNotifyType", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkNotifyType"), values);
   }
   return etype;
 }
@@ -295,7 +296,7 @@ gdk_crossing_mode_get_type (void)
       { GDK_CROSSING_UNGRAB, "GDK_CROSSING_UNGRAB", "ungrab" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkCrossingMode", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkCrossingMode"), values);
   }
   return etype;
 }
@@ -309,7 +310,7 @@ gdk_property_state_get_type (void)
       { GDK_PROPERTY_DELETE, "GDK_PROPERTY_DELETE", "delete" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkPropertyState", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkPropertyState"), values);
   }
   return etype;
 }
@@ -328,7 +329,7 @@ gdk_window_state_get_type (void)
       { GDK_WINDOW_STATE_BELOW, "GDK_WINDOW_STATE_BELOW", "below" },
       { 0, NULL, NULL }
     };
-    etype = g_flags_register_static ("GdkWindowState", values);
+    etype = g_flags_register_static (g_intern_static_string ("GdkWindowState"), values);
   }
   return etype;
 }
@@ -343,7 +344,7 @@ gdk_setting_action_get_type (void)
       { GDK_SETTING_ACTION_DELETED, "GDK_SETTING_ACTION_DELETED", "deleted" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkSettingAction", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkSettingAction"), values);
   }
   return etype;
 }
@@ -358,7 +359,7 @@ gdk_owner_change_get_type (void)
       { GDK_OWNER_CHANGE_CLOSE, "GDK_OWNER_CHANGE_CLOSE", "close" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkOwnerChange", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkOwnerChange"), values);
   }
   return etype;
 }
@@ -374,7 +375,7 @@ gdk_font_type_get_type (void)
       { GDK_FONT_FONTSET, "GDK_FONT_FONTSET", "fontset" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkFontType", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkFontType"), values);
   }
   return etype;
 }
@@ -392,7 +393,7 @@ gdk_cap_style_get_type (void)
       { GDK_CAP_PROJECTING, "GDK_CAP_PROJECTING", "projecting" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkCapStyle", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkCapStyle"), values);
   }
   return etype;
 }
@@ -408,7 +409,7 @@ gdk_fill_get_type (void)
       { GDK_OPAQUE_STIPPLED, "GDK_OPAQUE_STIPPLED", "opaque-stippled" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkFill", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkFill"), values);
   }
   return etype;
 }
@@ -436,7 +437,7 @@ gdk_function_get_type (void)
       { GDK_SET, "GDK_SET", "set" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkFunction", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkFunction"), values);
   }
   return etype;
 }
@@ -451,7 +452,7 @@ gdk_join_style_get_type (void)
       { GDK_JOIN_BEVEL, "GDK_JOIN_BEVEL", "bevel" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkJoinStyle", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkJoinStyle"), values);
   }
   return etype;
 }
@@ -466,7 +467,7 @@ gdk_line_style_get_type (void)
       { GDK_LINE_DOUBLE_DASH, "GDK_LINE_DOUBLE_DASH", "double-dash" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkLineStyle", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkLineStyle"), values);
   }
   return etype;
 }
@@ -480,7 +481,7 @@ gdk_subwindow_mode_get_type (void)
       { GDK_INCLUDE_INFERIORS, "GDK_INCLUDE_INFERIORS", "include-inferiors" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkSubwindowMode", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkSubwindowMode"), values);
   }
   return etype;
 }
@@ -510,7 +511,7 @@ gdk_gc_values_mask_get_type (void)
       { GDK_GC_JOIN_STYLE, "GDK_GC_JOIN_STYLE", "join-style" },
       { 0, NULL, NULL }
     };
-    etype = g_flags_register_static ("GdkGCValuesMask", values);
+    etype = g_flags_register_static (g_intern_static_string ("GdkGCValuesMask"), values);
   }
   return etype;
 }
@@ -527,7 +528,7 @@ gdk_image_type_get_type (void)
       { GDK_IMAGE_FASTEST, "GDK_IMAGE_FASTEST", "fastest" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkImageType", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkImageType"), values);
   }
   return etype;
 }
@@ -544,7 +545,7 @@ gdk_extension_mode_get_type (void)
       { GDK_EXTENSION_EVENTS_CURSOR, "GDK_EXTENSION_EVENTS_CURSOR", "cursor" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkExtensionMode", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkExtensionMode"), values);
   }
   return etype;
 }
@@ -560,7 +561,7 @@ gdk_input_source_get_type (void)
       { GDK_SOURCE_CURSOR, "GDK_SOURCE_CURSOR", "cursor" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkInputSource", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkInputSource"), values);
   }
   return etype;
 }
@@ -575,7 +576,7 @@ gdk_input_mode_get_type (void)
       { GDK_MODE_WINDOW, "GDK_MODE_WINDOW", "window" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkInputMode", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkInputMode"), values);
   }
   return etype;
 }
@@ -595,7 +596,7 @@ gdk_axis_use_get_type (void)
       { GDK_AXIS_LAST, "GDK_AXIS_LAST", "last" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkAxisUse", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkAxisUse"), values);
   }
   return etype;
 }
@@ -612,7 +613,7 @@ gdk_prop_mode_get_type (void)
       { GDK_PROP_MODE_APPEND, "GDK_PROP_MODE_APPEND", "append" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkPropMode", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkPropMode"), values);
   }
   return etype;
 }
@@ -628,7 +629,7 @@ gdk_fill_rule_get_type (void)
       { GDK_WINDING_RULE, "GDK_WINDING_RULE", "winding-rule" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkFillRule", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkFillRule"), values);
   }
   return etype;
 }
@@ -643,7 +644,7 @@ gdk_overlap_type_get_type (void)
       { GDK_OVERLAP_RECTANGLE_PART, "GDK_OVERLAP_RECTANGLE_PART", "part" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkOverlapType", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkOverlapType"), values);
   }
   return etype;
 }
@@ -660,7 +661,7 @@ gdk_rgb_dither_get_type (void)
       { GDK_RGB_DITHER_MAX, "GDK_RGB_DITHER_MAX", "max" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkRgbDither", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkRgbDither"), values);
   }
   return etype;
 }
@@ -676,7 +677,7 @@ gdk_byte_order_get_type (void)
       { GDK_MSB_FIRST, "GDK_MSB_FIRST", "msb-first" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkByteOrder", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkByteOrder"), values);
   }
   return etype;
 }
@@ -699,11 +700,14 @@ gdk_modifier_type_get_type (void)
       { GDK_BUTTON3_MASK, "GDK_BUTTON3_MASK", "button3-mask" },
       { GDK_BUTTON4_MASK, "GDK_BUTTON4_MASK", "button4-mask" },
       { GDK_BUTTON5_MASK, "GDK_BUTTON5_MASK", "button5-mask" },
+      { GDK_SUPER_MASK, "GDK_SUPER_MASK", "super-mask" },
+      { GDK_HYPER_MASK, "GDK_HYPER_MASK", "hyper-mask" },
+      { GDK_META_MASK, "GDK_META_MASK", "meta-mask" },
       { GDK_RELEASE_MASK, "GDK_RELEASE_MASK", "release-mask" },
       { GDK_MODIFIER_MASK, "GDK_MODIFIER_MASK", "modifier-mask" },
       { 0, NULL, NULL }
     };
-    etype = g_flags_register_static ("GdkModifierType", values);
+    etype = g_flags_register_static (g_intern_static_string ("GdkModifierType"), values);
   }
   return etype;
 }
@@ -718,7 +722,7 @@ gdk_input_condition_get_type (void)
       { GDK_INPUT_EXCEPTION, "GDK_INPUT_EXCEPTION", "exception" },
       { 0, NULL, NULL }
     };
-    etype = g_flags_register_static ("GdkInputCondition", values);
+    etype = g_flags_register_static (g_intern_static_string ("GdkInputCondition"), values);
   }
   return etype;
 }
@@ -735,7 +739,7 @@ gdk_status_get_type (void)
       { GDK_ERROR_MEM, "GDK_ERROR_MEM", "error-mem" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkStatus", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkStatus"), values);
   }
   return etype;
 }
@@ -752,7 +756,7 @@ gdk_grab_status_get_type (void)
       { GDK_GRAB_FROZEN, "GDK_GRAB_FROZEN", "frozen" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkGrabStatus", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkGrabStatus"), values);
   }
   return etype;
 }
@@ -772,7 +776,7 @@ gdk_visual_type_get_type (void)
       { GDK_VISUAL_DIRECT_COLOR, "GDK_VISUAL_DIRECT_COLOR", "direct-color" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkVisualType", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkVisualType"), values);
   }
   return etype;
 }
@@ -788,7 +792,7 @@ gdk_window_class_get_type (void)
       { GDK_INPUT_ONLY, "GDK_INPUT_ONLY", "only" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkWindowClass", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkWindowClass"), values);
   }
   return etype;
 }
@@ -806,7 +810,7 @@ gdk_window_type_get_type (void)
       { GDK_WINDOW_FOREIGN, "GDK_WINDOW_FOREIGN", "foreign" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkWindowType", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkWindowType"), values);
   }
   return etype;
 }
@@ -826,7 +830,7 @@ gdk_window_attributes_type_get_type (void)
       { GDK_WA_NOREDIR, "GDK_WA_NOREDIR", "noredir" },
       { 0, NULL, NULL }
     };
-    etype = g_flags_register_static ("GdkWindowAttributesType", values);
+    etype = g_flags_register_static (g_intern_static_string ("GdkWindowAttributesType"), values);
   }
   return etype;
 }
@@ -847,7 +851,7 @@ gdk_window_hints_get_type (void)
       { GDK_HINT_USER_SIZE, "GDK_HINT_USER_SIZE", "user-size" },
       { 0, NULL, NULL }
     };
-    etype = g_flags_register_static ("GdkWindowHints", values);
+    etype = g_flags_register_static (g_intern_static_string ("GdkWindowHints"), values);
   }
   return etype;
 }
@@ -865,9 +869,15 @@ gdk_window_type_hint_get_type (void)
       { GDK_WINDOW_TYPE_HINT_UTILITY, "GDK_WINDOW_TYPE_HINT_UTILITY", "utility" },
       { GDK_WINDOW_TYPE_HINT_DOCK, "GDK_WINDOW_TYPE_HINT_DOCK", "dock" },
       { GDK_WINDOW_TYPE_HINT_DESKTOP, "GDK_WINDOW_TYPE_HINT_DESKTOP", "desktop" },
+      { GDK_WINDOW_TYPE_HINT_DROPDOWN_MENU, "GDK_WINDOW_TYPE_HINT_DROPDOWN_MENU", "dropdown-menu" },
+      { GDK_WINDOW_TYPE_HINT_POPUP_MENU, "GDK_WINDOW_TYPE_HINT_POPUP_MENU", "popup-menu" },
+      { GDK_WINDOW_TYPE_HINT_TOOLTIP, "GDK_WINDOW_TYPE_HINT_TOOLTIP", "tooltip" },
+      { GDK_WINDOW_TYPE_HINT_NOTIFICATION, "GDK_WINDOW_TYPE_HINT_NOTIFICATION", "notification" },
+      { GDK_WINDOW_TYPE_HINT_COMBO, "GDK_WINDOW_TYPE_HINT_COMBO", "combo" },
+      { GDK_WINDOW_TYPE_HINT_DND, "GDK_WINDOW_TYPE_HINT_DND", "dnd" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkWindowTypeHint", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkWindowTypeHint"), values);
   }
   return etype;
 }
@@ -886,7 +896,7 @@ gdk_wm_decoration_get_type (void)
       { GDK_DECOR_MAXIMIZE, "GDK_DECOR_MAXIMIZE", "maximize" },
       { 0, NULL, NULL }
     };
-    etype = g_flags_register_static ("GdkWMDecoration", values);
+    etype = g_flags_register_static (g_intern_static_string ("GdkWMDecoration"), values);
   }
   return etype;
 }
@@ -904,7 +914,7 @@ gdk_wm_function_get_type (void)
       { GDK_FUNC_CLOSE, "GDK_FUNC_CLOSE", "close" },
       { 0, NULL, NULL }
     };
-    etype = g_flags_register_static ("GdkWMFunction", values);
+    etype = g_flags_register_static (g_intern_static_string ("GdkWMFunction"), values);
   }
   return etype;
 }
@@ -926,7 +936,7 @@ gdk_gravity_get_type (void)
       { GDK_GRAVITY_STATIC, "GDK_GRAVITY_STATIC", "static" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkGravity", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkGravity"), values);
   }
   return etype;
 }
@@ -946,7 +956,7 @@ gdk_window_edge_get_type (void)
       { GDK_WINDOW_EDGE_SOUTH_EAST, "GDK_WINDOW_EDGE_SOUTH_EAST", "south-east" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("GdkWindowEdge", values);
+    etype = g_enum_register_static (g_intern_static_string ("GdkWindowEdge"), values);
   }
   return etype;
 }
