@@ -193,8 +193,11 @@ _gdk_x11_events_uninit_screen (GdkScreen *screen)
 {
   GdkScreenX11 *screen_x11 = GDK_SCREEN_X11 (screen);
 
-  xsettings_client_destroy (screen_x11->xsettings_client);
-  screen_x11->xsettings_client = NULL;
+  if (screen_x11->xsettings_client)
+    {
+      xsettings_client_destroy (screen_x11->xsettings_client);
+      screen_x11->xsettings_client = NULL;
+    }
 }
 
 void 
@@ -222,11 +225,25 @@ _gdk_events_init (GdkDisplay *display)
   display_sources = g_list_prepend (display_sources,display_source);
 
   gdk_display_add_client_message_filter (display,
-					 gdk_atom_intern ("WM_PROTOCOLS", FALSE), 
+					 gdk_atom_intern_static_string ("WM_PROTOCOLS"), 
 					 gdk_wm_protocols_filter,   
 					 NULL);
 }
 
+void
+_gdk_events_uninit (GdkDisplay *display)
+{
+  GdkDisplayX11 *display_x11 = GDK_DISPLAY_X11 (display);
+
+  if (display_x11->event_source)
+    {
+      display_sources = g_list_remove (display_sources,
+                                       display_x11->event_source);
+      g_source_destroy (display_x11->event_source);
+      g_source_unref (display_x11->event_source);
+      display_x11->event_source = NULL;
+    }
+}
 
 /**
  * gdk_events_pending:
@@ -592,7 +609,7 @@ translate_key_event (GdkDisplay *display,
 {
   GdkKeymap *keymap = gdk_keymap_get_for_display (display);
   gunichar c = 0;
-  guchar buf[7];
+  gchar buf[7];
 
   event->key.type = xevent->xany.type == KeyPress ? GDK_KEY_PRESS : GDK_KEY_RELEASE;
   event->key.time = xevent->xkey.time;
@@ -610,6 +627,9 @@ translate_key_event (GdkDisplay *display,
 				       &event->key.keyval,
 				       NULL, NULL, NULL);
 
+  _gdk_keymap_add_virtual_modifiers (keymap, &event->key.state);
+  event->key.is_modifier = _gdk_keymap_key_is_modifier (keymap, event->key.hardware_keycode);
+
   /* Fill in event->string crudely, since various programs
    * depend on it.
    */
@@ -2047,7 +2067,7 @@ gdk_event_translate (GdkDisplay *display,
 	      break;
 	      
 	    case XkbStateNotify:
-	      _gdk_keymap_state_changed (display);
+	      _gdk_keymap_state_changed (display, xevent);
 	      break;
 	    }
 	}
@@ -2057,6 +2077,9 @@ gdk_event_translate (GdkDisplay *display,
       if (xevent->type - display_x11->xfixes_event_base == XFixesSelectionNotify)
 	{
 	  XFixesSelectionNotifyEvent *selection_notify = (XFixesSelectionNotifyEvent *)xevent;
+
+	  _gdk_x11_screen_process_owner_change (screen, xevent);
+	  
 	  event->owner_change.type = GDK_OWNER_CHANGE;
 	  event->owner_change.window = window;
 	  event->owner_change.owner = selection_notify->owner;
@@ -2066,7 +2089,7 @@ gdk_event_translate (GdkDisplay *display,
 					       selection_notify->selection);
 	  event->owner_change.time = selection_notify->timestamp;
 	  event->owner_change.selection_time = selection_notify->selection_timestamp;
-
+	  
 	  return_val = TRUE;
 	}
       else
@@ -2624,7 +2647,7 @@ gdk_x11_screen_get_window_manager_name (GdkScreen *screen)
           gint format;
           gulong n_items;
           gulong bytes_after;
-          guchar *name;
+          gchar *name;
           
           name = NULL;
 
@@ -2774,41 +2797,7 @@ gdk_net_wm_supports (GdkAtom property)
   return gdk_x11_screen_supports_net_wm_hint (gdk_screen_get_default (), property);
 }
 
-static const struct
-{
-  const char *xsettings_name;
-  const char *gdk_name;
-} settings_map[] = {
-  { "Net/DoubleClickTime", "gtk-double-click-time" },
-  { "Net/DoubleClickDistance", "gtk-double-click-distance" },
-  { "Net/DndDragThreshold", "gtk-dnd-drag-threshold" },
-  { "Net/CursorBlink", "gtk-cursor-blink" },
-  { "Net/CursorBlinkTime", "gtk-cursor-blink-time" },
-  { "Net/ThemeName", "gtk-theme-name" },
-  { "Net/IconThemeName", "gtk-icon-theme-name" },
-  { "Gtk/CanChangeAccels", "gtk-can-change-accels" },
-  { "Gtk/ColorPalette", "gtk-color-palette" },
-  { "Gtk/FontName", "gtk-font-name" },
-  { "Gtk/IconSizes", "gtk-icon-sizes" },
-  { "Gtk/KeyThemeName", "gtk-key-theme-name" },
-  { "Gtk/ToolbarStyle", "gtk-toolbar-style" },
-  { "Gtk/ToolbarIconSize", "gtk-toolbar-icon-size" },
-  { "Gtk/IMPreeditStyle", "gtk-im-preedit-style" },
-  { "Gtk/IMStatusStyle", "gtk-im-status-style" },
-  { "Gtk/Modules", "gtk-modules" },
-  { "Gtk/FileChooserBackend", "gtk-file-chooser-backend" },
-  { "Gtk/ButtonImages", "gtk-button-images" },
-  { "Gtk/MenuImages", "gtk-menu-images" },
-  { "Gtk/MenuBarAccel", "gtk-menu-bar-accel" },
-  { "Gtk/CursorThemeName", "gtk-cursor-theme-name" },
-  { "Gtk/CursorThemeSize", "gtk-cursor-theme-size" },
-  { "Xft/Antialias", "gtk-xft-antialias" },
-  { "Xft/Hinting", "gtk-xft-hinting" },
-  { "Xft/HintStyle", "gtk-xft-hintstyle" },
-  { "Xft/RGBA", "gtk-xft-rgba" },
-  { "Xft/DPI", "gtk-xft-dpi" },
-  { "Net/FallbackIconTheme", "gtk-fallback-icon-theme" }
-};
+#include "gdksettings.c"
 
 static void
 gdk_xsettings_notify_cb (const char       *name,
@@ -2829,10 +2818,10 @@ gdk_xsettings_notify_cb (const char       *name,
   new_event.setting.send_event = FALSE;
   new_event.setting.name = NULL;
 
-  for (i = 0; i < G_N_ELEMENTS (settings_map) ; i++)
-    if (strcmp (settings_map[i].xsettings_name, name) == 0)
+  for (i = 0; i < GDK_SETTINGS_N_ELEMENTS() ; i++)
+    if (strcmp (GDK_SETTINGS_X_NAME (i), name) == 0)
       {
-	new_event.setting.name = (char *)settings_map[i].gdk_name;
+	new_event.setting.name = (char*) GDK_SETTINGS_GDK_NAME (i);
 	break;
       }
   
@@ -2862,7 +2851,7 @@ check_transform (const gchar *xsettings_name,
 {
   if (!g_value_type_transformable (src_type, dest_type))
     {
-      g_warning ("Cannot tranform xsetting %s of type %s to type %s\n",
+      g_warning ("Cannot transform xsetting %s of type %s to type %s\n",
 		 xsettings_name,
 		 g_type_name (src_type),
 		 g_type_name (dest_type));
@@ -2907,10 +2896,10 @@ gdk_screen_get_setting (GdkScreen   *screen,
   
   screen_x11 = GDK_SCREEN_X11 (screen);
 
-  for (i = 0; i < G_N_ELEMENTS (settings_map) ; i++)
-    if (strcmp (settings_map[i].gdk_name, name) == 0)
+  for (i = 0; i < GDK_SETTINGS_N_ELEMENTS(); i++)
+    if (strcmp (GDK_SETTINGS_GDK_NAME (i), name) == 0)
       {
-	xsettings_name = settings_map[i].xsettings_name;
+	xsettings_name = GDK_SETTINGS_X_NAME (i);
 	break;
       }
 
