@@ -495,7 +495,7 @@ gtk_calendar_class_init (GtkCalendarClass *class)
 							 GTK_PARAM_READWRITE));
 
   gtk_calendar_signals[MONTH_CHANGED_SIGNAL] =
-    g_signal_new ("month_changed",
+    g_signal_new (I_("month_changed"),
 		  G_OBJECT_CLASS_TYPE (gobject_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkCalendarClass, month_changed),
@@ -503,7 +503,7 @@ gtk_calendar_class_init (GtkCalendarClass *class)
 		  _gtk_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
   gtk_calendar_signals[DAY_SELECTED_SIGNAL] =
-    g_signal_new ("day_selected",
+    g_signal_new (I_("day_selected"),
 		  G_OBJECT_CLASS_TYPE (gobject_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkCalendarClass, day_selected),
@@ -511,7 +511,7 @@ gtk_calendar_class_init (GtkCalendarClass *class)
 		  _gtk_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
   gtk_calendar_signals[DAY_SELECTED_DOUBLE_CLICK_SIGNAL] =
-    g_signal_new ("day_selected_double_click",
+    g_signal_new (I_("day_selected_double_click"),
 		  G_OBJECT_CLASS_TYPE (gobject_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkCalendarClass, day_selected_double_click),
@@ -519,7 +519,7 @@ gtk_calendar_class_init (GtkCalendarClass *class)
 		  _gtk_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
   gtk_calendar_signals[PREV_MONTH_SIGNAL] =
-    g_signal_new ("prev_month",
+    g_signal_new (I_("prev_month"),
 		  G_OBJECT_CLASS_TYPE (gobject_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkCalendarClass, prev_month),
@@ -527,7 +527,7 @@ gtk_calendar_class_init (GtkCalendarClass *class)
 		  _gtk_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
   gtk_calendar_signals[NEXT_MONTH_SIGNAL] =
-    g_signal_new ("next_month",
+    g_signal_new (I_("next_month"),
 		  G_OBJECT_CLASS_TYPE (gobject_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkCalendarClass, next_month),
@@ -535,7 +535,7 @@ gtk_calendar_class_init (GtkCalendarClass *class)
 		  _gtk_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
   gtk_calendar_signals[PREV_YEAR_SIGNAL] =
-    g_signal_new ("prev_year",
+    g_signal_new (I_("prev_year"),
 		  G_OBJECT_CLASS_TYPE (gobject_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkCalendarClass, prev_year),
@@ -543,7 +543,7 @@ gtk_calendar_class_init (GtkCalendarClass *class)
 		  _gtk_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
   gtk_calendar_signals[NEXT_YEAR_SIGNAL] =
-    g_signal_new ("next_year",
+    g_signal_new (I_("next_year"),
 		  G_OBJECT_CLASS_TYPE (gobject_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkCalendarClass, next_year),
@@ -689,7 +689,7 @@ gtk_calendar_init (GtkCalendar *calendar)
    * Do *not* translate it to anything else, if it
    * it isn't calendar:YM or calendar:MY it will not work.
    *
-   * Note that this flipping is in top the text direction flipping,
+   * Note that this flipping is in top of the text direction flipping,
    * so if you have a default text direction of RTL and YM, then
    * the year will appear on the right.
    */
@@ -812,12 +812,9 @@ static void
 calendar_set_year_next (GtkCalendar *calendar)
 {
   gint month_len;
-  GtkWidget *widget;
   
   g_return_if_fail (GTK_IS_WIDGET (calendar));
   
-  widget = GTK_WIDGET (calendar);
-  
   calendar->year++;
   calendar_compute_days (calendar);
   g_signal_emit (calendar,
@@ -1566,7 +1563,6 @@ gtk_calendar_size_request (GtkWidget	  *widget,
 
   gint height;
   gint i;
-  gchar buffer[255];
   gint calendar_margin = CALENDAR_MARGIN;
   gint header_width, main_width;
   gint max_header_height = 0;
@@ -1631,26 +1627,29 @@ gtk_calendar_size_request (GtkWidget	  *widget,
   /* Mainwindow labels width */
   
   priv->max_day_char_width = 0;
+  priv->max_day_char_ascent = 0;
+  priv->max_day_char_descent = 0;
   priv->min_day_width = 0;
-  priv->max_label_char_ascent = 0;
-  priv->max_label_char_descent = 0;
 
   for (i = 0; i < 9; i++)
     {
-      g_snprintf (buffer, sizeof (buffer), "%d%d", i, i);
+      gchar buffer[32];
+      g_snprintf (buffer, sizeof (buffer), Q_("calendar:day:digits|%d"), i * 11);
       pango_layout_set_text (layout, buffer, -1);	  
       pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
       priv->min_day_width = MAX (priv->min_day_width,
 					 logical_rect.width);
 
-      priv->max_day_char_ascent = MAX (priv->max_label_char_ascent,
+      priv->max_day_char_ascent = MAX (priv->max_day_char_ascent,
 					       PANGO_ASCENT (logical_rect));
-      priv->max_day_char_descent = MAX (priv->max_label_char_descent, 
+      priv->max_day_char_descent = MAX (priv->max_day_char_descent, 
 						PANGO_DESCENT (logical_rect));
     }
   /* We add one to max_day_char_width to be able to make the marked day "bold" */
   priv->max_day_char_width = priv->min_day_width / 2 + 1;
   
+  priv->max_label_char_ascent = 0;
+  priv->max_label_char_descent = 0;
   if (calendar->display_flags & GTK_CALENDAR_SHOW_DAY_NAMES)
     for (i = 0; i < 7; i++)
       {
@@ -1668,11 +1667,12 @@ gtk_calendar_size_request (GtkWidget	  *widget,
   if (calendar->display_flags & GTK_CALENDAR_SHOW_WEEK_NUMBERS)
     for (i = 0; i < 9; i++)
       {
-	g_snprintf (buffer, sizeof (buffer), "%d%d", i, i);
+	gchar buffer[32];
+	g_snprintf (buffer, sizeof (buffer), Q_("calendar:week:digits|%d"), i * 11);
 	pango_layout_set_text (layout, buffer, -1);	  
 	pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 	priv->max_week_char_width = MAX (priv->max_week_char_width,
-						 logical_rect.width / 2);
+					   logical_rect.width / 2);
       }
   
   main_width = (7 * (priv->min_day_width + (focus_padding + focus_width) * 2) + (DAY_XSEP * 6) + CALENDAR_MARGIN * 2
@@ -1730,16 +1730,10 @@ gtk_calendar_size_allocate (GtkWidget	  *widget,
   GtkCalendarPrivate *priv = GTK_CALENDAR_GET_PRIVATE (widget);
   gint xthickness = widget->style->xthickness;
   gint ythickness = widget->style->xthickness;
-  gboolean year_left;
   guint i;
   
   widget->allocation = *allocation;
-  
-  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR) 
-    year_left = priv->year_before;
-  else
-    year_left = !priv->year_before;
-  
+    
   if (calendar->display_flags & GTK_CALENDAR_SHOW_WEEK_NUMBERS)
     {
       priv->day_width = (priv->min_day_width
@@ -1841,7 +1835,7 @@ calendar_paint_header (GtkCalendar *calendar)
   cairo_t *cr;
   char buffer[255];
   int x, y;
-  gint header_width, cal_height;
+  gint header_width;
   gint max_month_width;
   gint max_year_width;
   PangoLayout *layout;
@@ -1859,7 +1853,6 @@ calendar_paint_header (GtkCalendar *calendar)
   cr = gdk_cairo_create (priv->header_win);
   
   header_width = widget->allocation.width - 2 * widget->style->xthickness;
-  cal_height = widget->allocation.height;
   
   max_month_width = priv->max_month_width;
   max_year_width = priv->max_year_width;
@@ -1948,7 +1941,6 @@ calendar_paint_day_names (GtkCalendar *calendar)
   char buffer[255];
   int day,i;
   int day_width, cal_width;
-  gint cal_height;
   int day_wid_sep;
   PangoLayout *layout;
   PangoRectangle logical_rect;
@@ -1964,7 +1956,6 @@ calendar_paint_day_names (GtkCalendar *calendar)
   
   day_width = priv->day_width;
   cal_width = widget->allocation.width;
-  cal_height = widget->allocation.height;
   day_wid_sep = day_width + DAY_XSEP;
   
   /*
@@ -2030,7 +2021,7 @@ calendar_paint_week_numbers (GtkCalendar *calendar)
   cairo_t *cr;
   gint row, week = 0, year;
   gint x_loc;
-  char buffer[3];
+  char buffer[32];
   gint y_loc, day_height;
   PangoLayout *layout;
   PangoRectangle logical_rect;
@@ -2084,7 +2075,18 @@ calendar_paint_week_numbers (GtkCalendar *calendar)
 			      + calendar->month) % 12 + 1, calendar->day[row][6]);
       g_return_if_fail (result);
 
-      g_snprintf (buffer, sizeof (buffer), "%d", week);
+      /* Translators: this defines whether the week numbers should use
+       * localized digits or the ones used in English (0123...).
+       *
+       * Translate to "%Id" if you want to use localized digits, or
+       * translate to "%d" otherwise.  Don't include the
+       * "calendar:week:digits|" part in the translation.
+       *
+       * Note that translating this doesn't guarantee that you get localized
+       * digits.  That needs support from your system and locale definition
+       * too.
+       */
+      g_snprintf (buffer, sizeof (buffer), Q_("calendar:week:digits|%d"), week);
       pango_layout_set_text (layout, buffer, -1);
       pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 
@@ -2150,7 +2152,7 @@ calendar_paint_day (GtkCalendar *calendar,
   GtkCalendarPrivate *priv = GTK_CALENDAR_GET_PRIVATE (calendar);
   cairo_t *cr;
   GdkColor *text_color;
-  gchar buffer[255];
+  gchar buffer[32];
   gint day;
   gint x_loc, y_loc;
   GdkRectangle day_rect;
@@ -2177,14 +2179,14 @@ calendar_paint_day (GtkCalendar *calendar,
     } 
   else 
     {
-      /*
+#if 0      
       if (calendar->highlight_row == row && calendar->highlight_col == col)
 	{
 	  cairo_set_source_color (cr, HIGHLIGHT_BG_COLOR (widget));
 	  gdk_cairo_rectangle (cr, &day_rect);
 	  cairo_fill (cr);
 	}
-      */
+#endif     
       if (calendar->selected_day == day)
 	{
 	  gdk_cairo_set_source_color (cr, SELECTED_BG_COLOR (widget));
@@ -2199,7 +2201,18 @@ calendar_paint_day (GtkCalendar *calendar,
 	text_color = NORMAL_DAY_COLOR (widget);
     }
 
-  g_snprintf (buffer, sizeof (buffer), "%d", day);
+  /* Translators: this defines whether the day numbers should use
+   * localized digits or the ones used in English (0123...).
+   *
+   * Translate to "%Id" if you want to use localized digits, or
+   * translate to "%d" otherwise.  Don't include the "calendar:day:digits|"
+   * part in the translation.
+   *
+   * Note that translating this doesn't guarantee that you get localized
+   * digits.  That needs support from your system and locale definition
+   * too.
+   */
+  g_snprintf (buffer, sizeof (buffer), Q_("calendar:day:digits|%d"), day);
   layout = gtk_widget_create_pango_layout (widget, buffer);
   pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
   
@@ -2230,8 +2243,11 @@ calendar_paint_day (GtkCalendar *calendar,
       
       gtk_paint_focus (widget->style, 
 		       priv->main_win,
+	               state,
+#if 0
 		       (calendar->selected_day == day) 
 		          ? GTK_STATE_SELECTED : GTK_STATE_NORMAL, 
+#endif
 		       NULL, widget, "calendar-day",
 		       day_rect.x,     day_rect.y, 
 		       day_rect.width, day_rect.height);
@@ -2339,9 +2355,6 @@ gtk_calendar_expose (GtkWidget	    *widget,
  *           Mouse handling             *
  ****************************************/
 
-#define CALENDAR_INITIAL_TIMER_DELAY    200
-#define CALENDAR_TIMER_DELAY            20
-
 static void
 calendar_arrow_action (GtkCalendar *calendar,
 		       guint        arrow)
@@ -2380,10 +2393,16 @@ calendar_timer (gpointer data)
 
       if (priv->need_timer)
 	{
+          GtkSettings *settings;
+          guint        timeout;
+
+          settings = gtk_widget_get_settings (GTK_WIDGET (calendar));
+          g_object_get (settings, "gtk-timeout-repeat", &timeout, NULL);
+
 	  priv->need_timer = FALSE;
-	  priv->timer = g_timeout_add (CALENDAR_TIMER_DELAY, 
-					       (GSourceFunc) calendar_timer, 
-					       (gpointer) calendar);
+	  priv->timer = g_timeout_add (timeout,
+                                       (GSourceFunc) calendar_timer,
+                                       (gpointer) calendar);
 	}
       else 
 	retval = TRUE;
@@ -2404,8 +2423,14 @@ calendar_start_spinning (GtkCalendar *calendar,
   
   if (!priv->timer)
     {
+      GtkSettings *settings;
+      guint        timeout;
+
+      settings = gtk_widget_get_settings (GTK_WIDGET (calendar));
+      g_object_get (settings, "gtk-timeout-initial", &timeout, NULL);
+
       priv->need_timer = TRUE;
-      priv->timer = g_timeout_add (CALENDAR_INITIAL_TIMER_DELAY, 
+      priv->timer = g_timeout_add (timeout,
 				   calendar_timer,
 				   calendar);
     }
@@ -2807,12 +2832,12 @@ gtk_calendar_key_press (GtkWidget   *widget,
     case GDK_space:
       row = calendar->focus_row;
       col = calendar->focus_col;
+      
       if (row > -1 && col > -1)
 	{
 	  return_val = TRUE;
-          
-          day = calendar->day[row][col];
 
+          day = calendar->day[row][col];
 	  if (calendar->day_month[row][col] == MONTH_PREV)
 	    calendar_set_month_prev (calendar);
 	  else if (calendar->day_month[row][col] == MONTH_NEXT)
@@ -2949,7 +2974,7 @@ set_status_pending (GdkDragContext *context,
                     GdkDragAction   suggested_action)
 {
   g_object_set_data (G_OBJECT (context),
-                     "gtk-calendar-status-pending",
+                     I_("gtk-calendar-status-pending"),
                      GINT_TO_POINTER (suggested_action));
 }
 
@@ -2989,7 +3014,7 @@ gtk_calendar_drag_motion (GtkWidget      *widget,
     }
   
   target = gtk_drag_dest_find_target (widget, context, NULL);
-  if (target == GDK_NONE || context->suggested_action == 0)
+  if (target == GDK_NONE)
     gdk_drag_status (context, 0, time);
   else
     {
@@ -3122,7 +3147,7 @@ gtk_calendar_new (void)
  *
  * Sets display options (whether to display the heading and the month headings).
  * 
- * Deprecated: Use gtk_calendar_set_display_options() instead
+ * Deprecated: 2.4: Use gtk_calendar_set_display_options() instead
  **/
 void
 gtk_calendar_display_options (GtkCalendar	       *calendar,
@@ -3468,6 +3493,8 @@ gtk_calendar_get_date (GtkCalendar *calendar,
  * 
  * Does nothing. Previously locked the display of the calendar until
  * it was thawed with gtk_calendar_thaw().
+ *
+ * Deprecated: 2.8: 
  **/
 void
 gtk_calendar_freeze (GtkCalendar *calendar)
@@ -3481,6 +3508,8 @@ gtk_calendar_freeze (GtkCalendar *calendar)
  * 
  * Does nothing. Previously defrosted a calendar; all the changes made
  * since the last gtk_calendar_freeze() were displayed.
+ *
+ * Deprecated: 2.8: 
  **/
 void
 gtk_calendar_thaw (GtkCalendar *calendar)
