@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-file-style: "gnu"; tab-width: 8 -*- */
 /* GTK - The GIMP Toolkit
  * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
  *
@@ -31,18 +32,14 @@
 #include <gdk/gdk.h>
 #include <gtk/gtkcontainer.h>
 
-
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 #define GTK_TYPE_NOTEBOOK                  (gtk_notebook_get_type ())
-#define GTK_NOTEBOOK(obj)                  (GTK_CHECK_CAST ((obj), GTK_TYPE_NOTEBOOK, GtkNotebook))
-#define GTK_NOTEBOOK_CLASS(klass)          (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_NOTEBOOK, GtkNotebookClass))
-#define GTK_IS_NOTEBOOK(obj)               (GTK_CHECK_TYPE ((obj), GTK_TYPE_NOTEBOOK))
-#define GTK_IS_NOTEBOOK_CLASS(klass)       (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_NOTEBOOK))
-#define GTK_NOTEBOOK_GET_CLASS(obj)        (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_NOTEBOOK, GtkNotebookClass))
+#define GTK_NOTEBOOK(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_NOTEBOOK, GtkNotebook))
+#define GTK_NOTEBOOK_CLASS(klass)          (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_NOTEBOOK, GtkNotebookClass))
+#define GTK_IS_NOTEBOOK(obj)               (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_NOTEBOOK))
+#define GTK_IS_NOTEBOOK_CLASS(klass)       (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_NOTEBOOK))
+#define GTK_NOTEBOOK_GET_CLASS(obj)        (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_NOTEBOOK, GtkNotebookClass))
 
 
 typedef enum
@@ -77,13 +74,18 @@ struct _GtkNotebook
   guint show_border        : 1;
   guint tab_pos            : 2;
   guint scrollable         : 1;
-  guint in_child           : 2;
-  guint click_child        : 2;
+  guint in_child           : 3;
+  guint click_child        : 3;
   guint button             : 2;
   guint need_timer         : 1;
   guint child_has_focus    : 1;
   guint have_visible_child : 1;
   guint focus_out          : 1;	/* Flag used by ::move-focus-out implementation */
+
+  guint has_before_previous : 1;
+  guint has_before_next     : 1;
+  guint has_after_previous  : 1;
+  guint has_after_next      : 1;
 };
 
 struct _GtkNotebookClass
@@ -103,33 +105,52 @@ struct _GtkNotebookClass
                                 gint               offset);
   void (* move_focus_out)      (GtkNotebook       *notebook,
 				GtkDirectionType   direction);
+  void (* reorder_tab)         (GtkNotebook       *notebook,
+				GtkDirectionType   direction,
+				gboolean           move_to_last);
+
+  /* More vfuncs */
+  gint (* insert_page)         (GtkNotebook       *notebook,
+			        GtkWidget         *child,
+				GtkWidget         *tab_label,
+				GtkWidget         *menu_label,
+				gint               position);
+
+  void (*_gtk_reserved1) (void);
+  void (*_gtk_reserved2) (void);
 };
 
+typedef GtkNotebook* (*GtkNotebookWindowCreationFunc) (GtkNotebook *source,
+                                                       GtkWidget   *page,
+                                                       gint         x,
+                                                       gint         y,
+                                                       gpointer     data);
+
 /***********************************************************
  *           Creation, insertion, deletion                 *
  ***********************************************************/
 
-GtkType gtk_notebook_get_type       (void) G_GNUC_CONST;
+GType   gtk_notebook_get_type       (void) G_GNUC_CONST;
 GtkWidget * gtk_notebook_new        (void);
-void gtk_notebook_append_page       (GtkNotebook *notebook,
+gint gtk_notebook_append_page       (GtkNotebook *notebook,
 				     GtkWidget   *child,
 				     GtkWidget   *tab_label);
-void gtk_notebook_append_page_menu  (GtkNotebook *notebook,
+gint gtk_notebook_append_page_menu  (GtkNotebook *notebook,
 				     GtkWidget   *child,
 				     GtkWidget   *tab_label,
 				     GtkWidget   *menu_label);
-void gtk_notebook_prepend_page      (GtkNotebook *notebook,
+gint gtk_notebook_prepend_page      (GtkNotebook *notebook,
 				     GtkWidget   *child,
 				     GtkWidget   *tab_label);
-void gtk_notebook_prepend_page_menu (GtkNotebook *notebook,
+gint gtk_notebook_prepend_page_menu (GtkNotebook *notebook,
 				     GtkWidget   *child,
 				     GtkWidget   *tab_label,
 				     GtkWidget   *menu_label);
-void gtk_notebook_insert_page       (GtkNotebook *notebook,
+gint gtk_notebook_insert_page       (GtkNotebook *notebook,
 				     GtkWidget   *child,
 				     GtkWidget   *tab_label,
 				     gint         position);
-void gtk_notebook_insert_page_menu  (GtkNotebook *notebook,
+gint gtk_notebook_insert_page_menu  (GtkNotebook *notebook,
 				     GtkWidget   *child,
 				     GtkWidget   *tab_label,
 				     GtkWidget   *menu_label,
@@ -137,6 +158,16 @@ void gtk_notebook_insert_page_menu  (GtkNotebook *notebook,
 void gtk_notebook_remove_page       (GtkNotebook *notebook,
 				     gint         page_num);
 
+/***********************************************************
+ *           Tabs drag and drop                            *
+ ***********************************************************/
+
+void gtk_notebook_set_window_creation_hook (GtkNotebookWindowCreationFunc  func,
+					    gpointer                       data);
+void gtk_notebook_set_group_id             (GtkNotebook *notebook,
+					    gint         group_id);
+gint gtk_notebook_get_group_id             (GtkNotebook *notebook);
+
 /***********************************************************
  *            query, set current NoteebookPage             *
  ***********************************************************/
@@ -144,6 +175,7 @@ void gtk_notebook_remove_page       (GtkNotebook *notebook,
 gint       gtk_notebook_get_current_page (GtkNotebook *notebook);
 GtkWidget* gtk_notebook_get_nth_page     (GtkNotebook *notebook,
 					  gint         page_num);
+gint       gtk_notebook_get_n_pages      (GtkNotebook *notebook);
 gint       gtk_notebook_page_num         (GtkNotebook *notebook,
 					  GtkWidget   *child);
 void       gtk_notebook_set_current_page (GtkNotebook *notebook,
@@ -224,15 +256,22 @@ void gtk_notebook_set_tab_label_packing   (GtkNotebook *notebook,
 void gtk_notebook_reorder_child           (GtkNotebook *notebook,
 					   GtkWidget   *child,
 					   gint         position);
+gboolean gtk_notebook_get_tab_reorderable (GtkNotebook *notebook,
+					   GtkWidget   *child);
+void gtk_notebook_set_tab_reorderable     (GtkNotebook *notebook,
+					   GtkWidget   *child,
+					   gboolean     reorderable);
+gboolean gtk_notebook_get_tab_detachable  (GtkNotebook *notebook,
+					   GtkWidget   *child);
+void gtk_notebook_set_tab_detachable      (GtkNotebook *notebook,
+					   GtkWidget   *child,
+					   gboolean     detachable);
 
 #ifndef GTK_DISABLE_DEPRECATED
 #define	gtk_notebook_current_page               gtk_notebook_get_current_page
 #define gtk_notebook_set_page                   gtk_notebook_set_current_page
 #endif /* GTK_DISABLE_DEPRECATED */
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
-
+G_END_DECLS
 
 #endif /* __GTK_NOTEBOOK_H__ */
