@@ -43,9 +43,10 @@ enum {
   PROP_DEFAULT_PAGE_SETUP,
   PROP_PRINT_SETTINGS,
   PROP_JOB_NAME,
-  PROP_NR_OF_PAGES,
+  PROP_N_PAGES,
   PROP_CURRENT_PAGE,
   PROP_USE_FULL_PAGE,
+  PROP_TRACK_PRINT_STATUS,
   PROP_UNIT,
   PROP_SHOW_DIALOG,
   PROP_PDF_TARGET,
@@ -118,6 +119,7 @@ gtk_print_operation_init (GtkPrintOperation *operation)
   priv->use_full_page = FALSE;
   priv->show_dialog = TRUE;
   priv->pdf_target = NULL;
+  priv->track_print_status = FALSE;
 
   priv->unit = GTK_UNIT_PIXEL;
 
@@ -144,8 +146,8 @@ gtk_print_operation_set_property (GObject      *object,
     case PROP_JOB_NAME:
       gtk_print_operation_set_job_name (op, g_value_get_string (value));
       break;
-    case PROP_NR_OF_PAGES:
-      gtk_print_operation_set_nr_of_pages (op, g_value_get_int (value));
+    case PROP_N_PAGES:
+      gtk_print_operation_set_n_pages (op, g_value_get_int (value));
       break;
     case PROP_CURRENT_PAGE:
       gtk_print_operation_set_current_page (op, g_value_get_int (value));
@@ -153,6 +155,9 @@ gtk_print_operation_set_property (GObject      *object,
     case PROP_USE_FULL_PAGE:
       gtk_print_operation_set_use_full_page (op, g_value_get_boolean (value));
       break;
+    case PROP_TRACK_PRINT_STATUS:
+      gtk_print_operation_set_track_print_status (op, g_value_get_boolean (value));
+      break;
     case PROP_UNIT:
       gtk_print_operation_set_unit (op, g_value_get_enum (value));
       break;
@@ -188,7 +193,7 @@ gtk_print_operation_get_property (GObject    *object,
     case PROP_JOB_NAME:
       g_value_set_string (value, priv->job_name);
       break;
-    case PROP_NR_OF_PAGES:
+    case PROP_N_PAGES:
       g_value_set_int (value, priv->nr_of_pages);
       break;
     case PROP_CURRENT_PAGE:
@@ -197,6 +202,9 @@ gtk_print_operation_get_property (GObject    *object,
     case PROP_USE_FULL_PAGE:
       g_value_set_boolean (value, priv->use_full_page);
       break;
+    case PROP_TRACK_PRINT_STATUS:
+      g_value_set_boolean (value, priv->track_print_status);
+      break;
     case PROP_UNIT:
       g_value_set_enum (value, priv->unit);
       break;
@@ -239,12 +247,12 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    *
    * A typical use for this signal is to use the parameters from the
    * #GtkPrintContext and paginate the document accordingly, and then
-   * set the number of pages with gtk_print_operation_set_nr_of_pages().
+   * set the number of pages with gtk_print_operation_set_n_pages().
    *
    * Since: 2.10
    */
   signals[BEGIN_PRINT] =
-    g_signal_new ("begin_print",
+    g_signal_new (I_("begin-print"),
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkPrintOperationClass, begin_print),
@@ -266,7 +274,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * Since: 2.10
    */
   signals[REQUEST_PAGE_SETUP] =
-    g_signal_new ("request_page_setup",
+    g_signal_new (I_("request-page-setup"),
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkPrintOperationClass, request_page_setup),
@@ -299,7 +307,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * Since: 2.10
    */
   signals[DRAW_PAGE] =
-    g_signal_new ("draw_page",
+    g_signal_new (I_("draw-page"),
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkPrintOperationClass, draw_page),
@@ -321,7 +329,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * Since: 2.10
    */
   signals[END_PRINT] =
-    g_signal_new ("end_print",
+    g_signal_new (I_("end-print"),
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkPrintOperationClass, end_print),
@@ -341,13 +349,13 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * Since: 2.10
    */
   signals[STATUS_CHANGED] =
-   g_signal_new ("status-changed",
-                 G_TYPE_FROM_CLASS (class),
-                 G_SIGNAL_RUN_LAST,
-                 G_STRUCT_OFFSET (GtkPrintOperationClass, status_changed),
-                 NULL, NULL,
-                 g_cclosure_marshal_VOID__VOID,
-                 G_TYPE_NONE, 0);
+    g_signal_new (I_("status-changed"),
+		     G_TYPE_FROM_CLASS (class),
+		     G_SIGNAL_RUN_LAST,
+		     G_STRUCT_OFFSET (GtkPrintOperationClass, status_changed),
+		     NULL, NULL,
+		     g_cclosure_marshal_VOID__VOID,
+		     G_TYPE_NONE, 0);
 
   /**
    * GtkPrintOperation:default-page-setup:
@@ -373,8 +381,9 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    *
    * The #GtkPrintSettings used for initializing the dialog.
    *
-   * Setting this property is typically used to re-establish print 
-   * settings from a previous print operation, see gtk_print_operation_run().
+   * Setting this property is typically used to re-establish 
+   * print settings from a previous print operation, see 
+   * gtk_print_operation_run().
    *
    * Since: 2.10
    */
@@ -389,10 +398,11 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
   /**
    * GtkPrintOperation:job-name:
    *
-   * A string used to identify the job (e.g. in monitoring applications like eggcups). 
+   * A string used to identify the job (e.g. in monitoring 
+   * applications like eggcups). 
    * 
-   * If you don't set a job name, GTK+ picks a default one by numbering successive 
-   * print jobs.
+   * If you don't set a job name, GTK+ picks a default one 
+   * by numbering successive print jobs.
    *
    * Since: 2.10
    */
@@ -405,22 +415,24 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
 							GTK_PARAM_READWRITE));
 
   /**
-   * GtkPrintOperation:number-of-pages:
+   * GtkPrintOperation:n-pages:
    *
    * The number of pages in the document. 
    *
    * This <emphasis>must</emphasis> be set to a positive number
-   * before the rendering starts. It may be set in a ::begin-print signal hander.
+   * before the rendering starts. It may be set in a ::begin-print 
+   * signal hander.
    *
-   * Note that the page numbers passed to the ::request-page-setup and ::draw-page 
-   * signals are 0-based, i.e. if the user chooses to print all pages, the last 
-   * ::draw-page signal will be for page @n_pages - 1.
+   * Note that the page numbers passed to the ::request-page-setup 
+   * and ::draw-page signals are 0-based, i.e. if the user chooses 
+   * to print all pages, the last ::draw-page signal will be for 
+   * page @n_pages - 1.
    *
    * Since: 2.10
    */
   g_object_class_install_property (gobject_class,
-				   PROP_NR_OF_PAGES,
-				   g_param_spec_int ("number-of-pages",
+				   PROP_N_PAGES,
+				   g_param_spec_int ("n-pages",
 						     P_("Number of Pages"),
 						     P_("The number of pages in the document."),
 						     -1,
@@ -453,11 +465,12 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
   /**
    * GtkPrintOperation:use-full-page:
    *
-   * If %TRUE, the transformation for the cairo context obtained from 
-   * #GtkPrintContext puts the origin at the top left corner of the page 
-   * (which may not be the top left corner of the sheet, depending on page 
-   * orientation and the number of pages per sheet). Otherwise, the origin 
-   * is at the top left corner of the imageable area (i.e. inside the margins).
+   * If %TRUE, the transformation for the cairo context obtained 
+   * from #GtkPrintContext puts the origin at the top left corner 
+   * of the page (which may not be the top left corner of the sheet, 
+   * depending on page orientation and the number of pages per sheet). 
+   * Otherwise, the origin is at the top left corner of the imageable 
+   * area (i.e. inside the margins).
    * 
    * Since: 2.10
    */
@@ -465,7 +478,28 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
 				   PROP_USE_FULL_PAGE,
 				   g_param_spec_boolean ("use-full-page",
 							 P_("Use full page"),
-							 P_("%TRUE if the the origin of the context should be at the corner of the page and not the corner of the imageable area"),
+							 P_("TRUE if the the origin of the context should be at the corner of the page and not the corner of the imageable area"),
+							 FALSE,
+							 GTK_PARAM_READWRITE));
+  
+
+  /**
+   * GtkPrintOperation:track-print-status:
+   *
+   * If %TRUE, the print operation will try to continue report on 
+   * the status of the print job in the printer queues and printer. 
+   * This can allow your application to show things like "out of paper" 
+   * issues, and when the print job actually reaches the printer. 
+   * However, this is often implemented using polling, and should 
+   * not be enabled unless needed.
+   * 
+   * Since: 2.10
+   */
+  g_object_class_install_property (gobject_class,
+				   PROP_USE_FULL_PAGE,
+				   g_param_spec_boolean ("track-print-status",
+							 P_("Track Print Status"),
+							 P_("TRUE if the print operation will continue to report on the print job status after the print data has been sent to the printer or print server."),
 							 FALSE,
 							 GTK_PARAM_READWRITE));
   
@@ -474,8 +508,8 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * GtkPrintOperation:unit:
    *
    * The transformation for the cairo context obtained from
-   * #GtkPrintContext is set up in such a way that distances are measured 
-   * in units of @unit.
+   * #GtkPrintContext is set up in such a way that distances 
+   * are measured in units of @unit.
    *
    * Since: 2.10
    */
@@ -501,21 +535,22 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
 				   PROP_SHOW_DIALOG,
 				   g_param_spec_boolean ("show-dialog",
 							 P_("Show Dialog"),
-							 P_("%TRUE if gtk_print_operation_run() should show the print dialog."),
+							 P_("TRUE if gtk_print_operation_run() should show the print dialog."),
 							 TRUE,
 							 GTK_PARAM_READWRITE));
   
   /**
    * GtkPrintOperation:pdf-target:
    *
-   * The name of a PDF file to generate instead of showing the print dialog. 
+   * The name of a PDF file to generate instead of showing 
+   * the print dialog. 
    *
-   * The indended use of this property is for implementing "Export to PDF" 
-   * actions.
+   * The intended use of this property is for implementing 
+   * "Export to PDF" actions.
    *
    * "Print to PDF" support is independent of this and is done
-   * by letting the user pick the "Print to PDF" item from the list
-   * of printers in the print dialog.
+   * by letting the user pick the "Print to PDF" item from the 
+   * list of printers in the print dialog.
    *
    * Since: 2.10
    */
@@ -523,7 +558,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
 				   PROP_JOB_NAME,
 				   g_param_spec_string ("pdf-target",
 							P_("PDF target filename"),
-							P_(""),
+							P_("PDF target filename"),
 							NULL,
 							GTK_PARAM_READWRITE));
   
@@ -734,7 +769,7 @@ gtk_print_operation_set_job_name (GtkPrintOperation *op,
 }
 
 /**
- * gtk_print_operation_set_nr_of_pages:
+ * gtk_print_operation_set_n_pages:
  * @op: a #GtkPrintOperation
  * @n_pages: the number of pages
  * 
@@ -752,8 +787,8 @@ gtk_print_operation_set_job_name (GtkPrintOperation *op,
  * Since: 2.10
  **/
 void
-gtk_print_operation_set_nr_of_pages (GtkPrintOperation *op,
-				     gint               n_pages)
+gtk_print_operation_set_n_pages (GtkPrintOperation *op,
+				 gint               n_pages)
 {
   GtkPrintOperationPrivate *priv;
 
@@ -768,7 +803,7 @@ gtk_print_operation_set_nr_of_pages (GtkPrintOperation *op,
     {
       priv->nr_of_pages = n_pages;
 
-      g_object_notify (G_OBJECT (op), "number-of-pages");
+      g_object_notify (G_OBJECT (op), "n-pages");
     }
 }
 
@@ -870,6 +905,40 @@ gtk_print_operation_set_unit (GtkPrintOperation *op,
     }
 }
 
+/**
+ * gtk_print_operation_set_track_print_status:
+ * @op: a #GtkPrintOperation
+ * @track_status: %TRUE to track status after printing
+ * 
+ * If track_status is %TRUE, the print operation will try to continue report
+ * on the status of the print job in the printer queues and printer. This
+ * can allow your application to show things like "out of paper" issues,
+ * and when the print job actually reaches the printer.
+ * 
+ * This function is often implemented using some form of polling, so it should
+ * not be enabled unless needed.
+ *
+ * Since: 2.10
+ */
+void
+gtk_print_operation_set_track_print_status (GtkPrintOperation  *op,
+					    gboolean            track_status)
+{
+  GtkPrintOperationPrivate *priv;
+
+  g_return_if_fail (GTK_IS_PRINT_OPERATION (op));
+
+  priv = op->priv;
+
+  if (priv->track_print_status != track_status)
+    {
+      priv->track_print_status = track_status;
+
+      g_object_notify (G_OBJECT (op), "track-print-status");
+    }
+}
+
+
 void
 _gtk_print_operation_set_status (GtkPrintOperation *op,
 				 GtkPrintStatus     status,
@@ -1124,7 +1193,7 @@ pdf_end_page (GtkPrintOperation *op,
 
 static void
 pdf_end_run (GtkPrintOperation *op,
-	     gboolean wait)
+	     gboolean           wait)
 {
   GtkPrintOperationPrivate *priv = op->priv;
 
@@ -1176,20 +1245,155 @@ run_pdf (GtkPrintOperation  *op,
   return GTK_PRINT_OPERATION_RESULT_APPLY; 
 }
 
-static void
-print_pages (GtkPrintOperation *op,
-	     gboolean wait)
+typedef struct
 {
-  GtkPrintOperationPrivate *priv = op->priv;
-  int page, range;
-  GtkPageSetup *initial_page_setup, *page_setup;
-  GtkPrintContext *print_context;
-  cairo_t *cr;
-  int uncollated_copies, collated_copies;
-  int i, j;
+  GtkPrintOperation *op;
+  gint uncollated_copies;
+  gint collated_copies;
+  gint uncollated, collated;
+
+  gint range, num_ranges;
   GtkPageRange *ranges;
   GtkPageRange one_range;
+
+  gint page, start, end, inc;
+
+  GtkPageSetup *initial_page_setup;
+  GtkPrintContext *print_context;  
+} PrintPagesData;
+
+static void
+find_range (PrintPagesData *data)
+{
+  GtkPageRange *range;
+
+  range = &data->ranges[data->range];
+
+  if (data->inc < 0)
+    {
+      data->start = range->end;
+      data->end = range->start - 1;
+    }
+  else
+    {
+      data->start = range->start;
+      data->end = range->end + 1;
+    }
+}
+
+static gboolean 
+increment_page_sequence (PrintPagesData *data)
+{
+  GtkPrintOperationPrivate *priv = data->op->priv;
+
+  do {
+    data->page += data->inc;
+    if (data->page == data->end)
+      {
+	data->range += data->inc;
+	if (data->range == -1 || data->range == data->num_ranges)
+	  {
+	    data->uncollated++;
+	    if (data->uncollated == data->uncollated_copies)
+	      return FALSE;
+
+	    data->range = data->inc < 0 ? data->num_ranges - 1 : 0;
+	  }
+	find_range (data);
+	data->page = data->start;
+      }
+  }
+  while ((priv->manual_page_set == GTK_PAGE_SET_EVEN && data->page % 2 == 0) ||
+	 (priv->manual_page_set == GTK_PAGE_SET_ODD && data->page % 2 == 1));
+
+  return TRUE;
+}
+
+static gboolean
+print_pages_idle (gpointer user_data)
+{
+  PrintPagesData *data; 
+  GtkPrintOperationPrivate *priv; 
+  GtkPageSetup *page_setup;
+  cairo_t *cr;
+  gboolean done = FALSE;
+
+  GDK_THREADS_ENTER ();
+
+  data = (PrintPagesData*)user_data;
+  priv = data->op->priv;
+
+  data->collated++;
+  if (data->collated == data->collated_copies)
+    {
+      data->collated = 0;
+      if (!increment_page_sequence (data))
+	{
+	  g_signal_emit (data->op, signals[END_PRINT], 0, data->print_context);
+	  
+	  g_object_unref (data->print_context);
+	  g_object_unref (data->initial_page_setup);
+   
+	  cairo_surface_finish (data->op->priv->surface);
+	  priv->end_run (data->op, TRUE);
+	  
+	  g_object_unref (data->op);
+	  g_free (data);
+
+	  done = TRUE;
+
+	  goto out;
+	}
+    }
+
+  page_setup = gtk_page_setup_copy (data->initial_page_setup);
+  g_signal_emit (data->op, signals[REQUEST_PAGE_SETUP], 0, 
+		 data->print_context, data->page, page_setup);
+  
+  _gtk_print_context_set_page_setup (data->print_context, page_setup);
+  data->op->priv->start_page (data->op, data->print_context, page_setup);
+  
+  cr = gtk_print_context_get_cairo (data->print_context);
+  
+  cairo_save (cr);
+  if (priv->manual_scale != 100.0)
+    cairo_scale (cr,
+		 priv->manual_scale,
+		 priv->manual_scale);
+  
+  if (priv->manual_orientation)
+    _gtk_print_context_rotate_according_to_orientation (data->print_context);
+  
+  if (!priv->use_full_page)
+    _gtk_print_context_translate_into_margin (data->print_context);
+  
+  g_signal_emit (data->op, signals[DRAW_PAGE], 0, 
+		 data->print_context, data->page);
+  
+  priv->end_page (data->op, data->print_context);
+  
+  cairo_restore (cr);
+  
+  g_object_unref (page_setup);
+
+ out:
+
+  GDK_THREADS_LEAVE ();
+
+  return !done;
+}
+  
+static void
+print_pages (GtkPrintOperation *op,
+	     gboolean           wait)
+{
+  GtkPrintOperationPrivate *priv = op->priv;
+  GtkPageSetup *initial_page_setup;
+  GtkPrintContext *print_context;
+  int uncollated_copies, collated_copies;
+  GtkPageRange *ranges;
   int num_ranges;
+  PrintPagesData *data;
 
   if (priv->manual_collation)
     {
@@ -1207,7 +1411,7 @@ print_pages (GtkPrintOperation *op,
   initial_page_setup = create_page_setup (op);
   _gtk_print_context_set_page_setup (print_context, initial_page_setup);
 
-  _gtk_print_operation_set_status (op, GTK_PRINT_STATUS_PREPARING, NULL);
+  _gtk_print_operation_set_status (op, GTK_PRINT_STATUS_PREPARING, NULL);  
   g_signal_emit (op, signals[BEGIN_PRINT], 0, print_context);
   
   g_return_if_fail (priv->nr_of_pages > 0);
@@ -1220,14 +1424,14 @@ print_pages (GtkPrintOperation *op,
   else if (priv->print_pages == GTK_PRINT_PAGES_CURRENT &&
 	   priv->current_page != -1)
     {
-      ranges = &one_range;
+      ranges = &data->one_range;
       num_ranges = 1;
       ranges[0].start = priv->current_page;
       ranges[0].end = priv->current_page;
     }
   else
     {
-      ranges = &one_range;
+      ranges = &data->one_range;
       num_ranges = 1;
       ranges[0].start = 0;
       ranges[0].end = priv->nr_of_pages - 1;
@@ -1235,78 +1439,43 @@ print_pages (GtkPrintOperation *op,
   
   _gtk_print_operation_set_status (op, GTK_PRINT_STATUS_GENERATING_DATA, NULL);
 
-  for (i = 0; i < uncollated_copies; i++)
+  data = g_new0 (PrintPagesData, 1);
+  data->op = g_object_ref (op);
+  data->uncollated_copies = uncollated_copies;
+  data->collated_copies = collated_copies;
+  data->ranges = ranges;
+  data->num_ranges = num_ranges;
+  data->initial_page_setup = initial_page_setup;
+  data->print_context = print_context;
+
+  data->uncollated = 0; 
+  if (data->op->priv->manual_reverse)
     {
-      for (range = 0; range < num_ranges; range ++)
+      data->range = data->num_ranges - 1;
+      data->inc = -1;      
+    }
+  else
+    {
+      data->range = 0;
+      data->inc = 1;      
+    }
+  find_range (data);
+
+  data->page = data->start - data->inc;
+  data->collated = data->collated_copies - 1;
+
+  if (wait)
+    {
+      /* FIXME replace this with a recursive mainloop */
+      while (print_pages_idle (data))
 	{
-	  int start, end, inc;
-	  int low = ranges[range].start;
-	  int high = ranges[range].end;
-	  
-	  if (priv->manual_reverse)
-	    {
-	      start = high;
-	      end = low - 1;
-	      inc = -1;
-	    }
-	  else
-	    {
-	      start = low;
-	      end = high + 1;
-	      inc = 1;
-	    }
-	  for (page = start; page != end; page += inc)
-	    {
-	      if ((priv->manual_page_set == GTK_PAGE_SET_EVEN && page % 2 == 0) ||
-		  (priv->manual_page_set == GTK_PAGE_SET_ODD && page % 2 == 1))
-		continue;
-	      
-	      for (j = 0; j < collated_copies; j++)
-		{
-		  page_setup = gtk_page_setup_copy (initial_page_setup);
-		  g_signal_emit (op, signals[REQUEST_PAGE_SETUP], 0, print_context, page, page_setup);
-		  
-		  _gtk_print_context_set_page_setup (print_context, page_setup);
-		  priv->start_page (op, print_context, page_setup);
-		  
-		  cr = gtk_print_context_get_cairo (print_context);
-
-		  cairo_save (cr);
-		  if (priv->manual_scale != 100.0)
-		    cairo_scale (cr,
-				 priv->manual_scale,
-				 priv->manual_scale);
-		  
-		  if (priv->manual_orientation)
-		    _gtk_print_context_rotate_according_to_orientation (print_context);
-
-		  if (!priv->use_full_page)
-		    _gtk_print_context_translate_into_margin (print_context);
-	  
-		  g_signal_emit (op, signals[DRAW_PAGE], 0, 
-				 print_context, page);
-		  
-		  priv->end_page (op, print_context);
-		  
-		  cairo_restore (cr);
-		  
-		  g_object_unref (page_setup);
-
-		  /* Iterate the mainloop so that we redraw windows */
-		  while (gtk_events_pending ())
-		    gtk_main_iteration ();
-		}
-	    }
+	  /* Iterate the mainloop so that we redraw windows */
+	  while (gtk_events_pending ())
+	    gtk_main_iteration ();
 	}
     }
-  
-  g_signal_emit (op, signals[END_PRINT], 0, print_context);
-
-  g_object_unref (print_context);
-  g_object_unref (initial_page_setup);
-
-  cairo_surface_finish (priv->surface);
-  priv->end_run (op, wait);
+  else
+    g_idle_add (print_pages_idle, data);
 }
 
 /**
@@ -1330,7 +1499,7 @@ print_pages (GtkPrintOperation *op,
  * </programlisting></informalexample>
  *
  * Return value: the result of the print operation. A return value of 
- *   %GTK_PRINT_OPERATION_RESULT_APPLY indicates that the printing was 
+ *   %GTK_PRINT_OPERATION_RESULT_APPLY indicates that the printing was
  *   completed successfully. In this case, it is a good idea to obtain 
  *   the used print settings with gtk_print_operation_get_print_settings() 
  *   and store them for reuse with the next print operation.
