@@ -24,10 +24,11 @@
 #include <gtk/gtktreemodel.h>
 #include <gtk/gtktreeviewcolumn.h>
 #include <gtk/gtkdnd.h>
+#include <gtk/gtkentry.h>
+
+
+G_BEGIN_DECLS
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
 
 typedef enum
 {
@@ -127,6 +128,9 @@ typedef gboolean (*GtkTreeViewSearchEqualFunc) (GtkTreeModel            *model,
 typedef gboolean (*GtkTreeViewRowSeparatorFunc) (GtkTreeModel      *model,
 						 GtkTreeIter       *iter,
 						 gpointer           data);
+typedef void     (*GtkTreeViewSearchPositionFunc) (GtkTreeView  *tree_view,
+						   GtkWidget    *search_dialog,
+						   gpointer      user_data);
 
 
 /* Creators */
@@ -149,6 +153,7 @@ gboolean               gtk_tree_view_get_headers_visible           (GtkTreeView
 void                   gtk_tree_view_set_headers_visible           (GtkTreeView               *tree_view,
 								    gboolean                   headers_visible);
 void                   gtk_tree_view_columns_autosize              (GtkTreeView               *tree_view);
+gboolean               gtk_tree_view_get_headers_clickable         (GtkTreeView *tree_view);
 void                   gtk_tree_view_set_headers_clickable         (GtkTreeView               *tree_view,
 								    gboolean                   setting);
 void                   gtk_tree_view_set_rules_hint                (GtkTreeView               *tree_view,
@@ -262,6 +267,9 @@ void                   gtk_tree_view_tree_to_widget_coords         (GtkTreeView
 								    gint                       ty,
 								    gint                      *wx,
 								    gint                      *wy);
+gboolean               gtk_tree_view_get_visible_range             (GtkTreeView               *tree_view,
+								    GtkTreePath              **start_path,
+								    GtkTreePath              **end_path);
 
 /* Drag-and-Drop support */
 void                   gtk_tree_view_enable_model_drag_source      (GtkTreeView               *tree_view,
@@ -305,6 +313,15 @@ void                       gtk_tree_view_set_search_equal_func (GtkTreeView
 								gpointer                    search_user_data,
 								GtkDestroyNotify            search_destroy);
 
+GtkEntry                     *gtk_tree_view_get_search_entry         (GtkTreeView                   *tree_view);
+void                          gtk_tree_view_set_search_entry         (GtkTreeView                   *tree_view,
+								      GtkEntry                      *entry);
+GtkTreeViewSearchPositionFunc gtk_tree_view_get_search_position_func (GtkTreeView                   *tree_view);
+void                          gtk_tree_view_set_search_position_func (GtkTreeView                   *tree_view,
+								      GtkTreeViewSearchPositionFunc  func,
+								      gpointer                       data,
+								      GDestroyNotify                 destroy);
+
 /* This function should really never be used.  It is just for use by ATK.
  */
 typedef void (* GtkTreeDestroyCountFunc)  (GtkTreeView             *tree_view,
@@ -332,9 +349,8 @@ void                        gtk_tree_view_set_row_separator_func (GtkTreeView
 								  gpointer                    data,
 								  GtkDestroyNotify            destroy);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+
+G_END_DECLS
 
 
 #endif /* __GTK_TREE_VIEW_H__ */
