@@ -1317,6 +1317,7 @@ gtk_message_type_get_type (void)
       { GTK_MESSAGE_WARNING, "GTK_MESSAGE_WARNING", "warning" },
       { GTK_MESSAGE_QUESTION, "GTK_MESSAGE_QUESTION", "question" },
       { GTK_MESSAGE_ERROR, "GTK_MESSAGE_ERROR", "error" },
+      { GTK_MESSAGE_OTHER, "GTK_MESSAGE_OTHER", "other" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static (g_intern_static_string ("GtkMessageType"), values);
@@ -1634,10 +1635,8 @@ gtk_recent_manager_error_get_type (void)
     static const GEnumValue values[] = {
       { GTK_RECENT_MANAGER_ERROR_NOT_FOUND, "GTK_RECENT_MANAGER_ERROR_NOT_FOUND", "not-found" },
       { GTK_RECENT_MANAGER_ERROR_INVALID_URI, "GTK_RECENT_MANAGER_ERROR_INVALID_URI", "invalid-uri" },
-      { GTK_RECENT_MANAGER_ERROR_INVALID_MIME, "GTK_RECENT_MANAGER_ERROR_INVALID_MIME", "invalid-mime" },
       { GTK_RECENT_MANAGER_ERROR_INVALID_ENCODING, "GTK_RECENT_MANAGER_ERROR_INVALID_ENCODING", "invalid-encoding" },
       { GTK_RECENT_MANAGER_ERROR_NOT_REGISTERED, "GTK_RECENT_MANAGER_ERROR_NOT_REGISTERED", "not-registered" },
-      { GTK_RECENT_MANAGER_ERROR_BAD_EXEC_STRING, "GTK_RECENT_MANAGER_ERROR_BAD_EXEC_STRING", "bad-exec-string" },
       { GTK_RECENT_MANAGER_ERROR_READ, "GTK_RECENT_MANAGER_ERROR_READ", "read" },
       { GTK_RECENT_MANAGER_ERROR_WRITE, "GTK_RECENT_MANAGER_ERROR_WRITE", "write" },
       { GTK_RECENT_MANAGER_ERROR_UNKNOWN, "GTK_RECENT_MANAGER_ERROR_UNKNOWN", "unknown" },
